﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TFC.Models;
using TFC.Helpers;

namespace TFC.Controllers
{
    public class AnunciosController : BaseController
    {
        private TFCEntities1 db = new TFCEntities1();

        //
        // GET: /Anuncios/

        public ActionResult Index(bool privados = false)
        {
            IEnumerable<Anuncio> anuncios = null;

            if (privados == true)
            {
                // Obtenemos las comunidades del usuario conectado
                IList<Comunidad> comunidades = Datos.GetComunidades(User.Identity.Name);

                // Anuncios privados
                if (User.IsInRole("Propietario"))
                {
                    // Si es un propietario, mostramos los anuncios privados de las comunidades donde tiene alguna propiedad
                    anuncios = (from co in db.Comunicaciones.OfType<Anuncio>()
                                                                .Include("CategoriaComunicacion")
                                join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                                join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                                join p in db.Propiedades on c.IdComunidad equals p.IdComunidad
                                join pp in db.PropiedadesPropietarios on p.IdPropiedad equals pp.IdPropiedad
                                join po in db.Personas.OfType<Propietario>() on pp.PropietarioIdPersona equals po.IdPersona
                                where po.Email == User.Identity.Name && co.Privada == true
                                orderby co.Fecha descending
                                select co).Distinct();

                }
                else if (User.IsInRole("Administrador de fincas"))
                {
                    // Si es un administrador de fincas, mostramos todos los anuncios privados de sus comunidades.
                    anuncios = from co in db.Comunicaciones.OfType<Anuncio>()
                                        .Include("CategoriaComunicacion")
                               join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                               join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad 
                               join af in db.Personas.OfType<AdministradorFincas>() on c.AdministradorFincasIdPersona equals af.IdPersona
                               where af.Email == User.Identity.Name && co.Privada == true
                               orderby co.Fecha descending
                               select co;
                }
            }
            else
            {
                // Anuncios públicos. Son visibles por todos los usuarios de la aplicación.
                anuncios = from co in db.Comunicaciones.OfType<Anuncio>()
                                        .Include("CategoriaComunicacion")
                           where co.Privada == false
                           orderby co.Fecha descending
                           select co;
            }

            ViewBag.Privados = privados;
            ViewBag.IdPersona = Entorno.PersonaConectada(User.Identity.Name).IdPersona;
            return View(anuncios.ToList());
        }

        //
        // GET: /Anuncios/Details/5

        public ActionResult Details(int id = 0)
        {
            Anuncio anuncio = db.Comunicaciones.OfType<Anuncio>().Single(a => a.IdComunicacion == id);
            if (anuncio == null)
            {
                return HttpNotFound();
            }
            return View(anuncio);
        }

        //
        // GET: /Anuncios/Create

        public ActionResult Create(bool privado)
        {
            ViewBag.Privado = privado;
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion");
            
            return View();
        }

        //
        // POST: /Anuncios/Create

        [HttpPost]
        public ActionResult Create(Anuncio anuncio)
        {
            anuncio.IdPersona = Entorno.PersonaConectada(User.Identity.Name).IdPersona;
            anuncio.Fecha = DateTime.Now;

            if (ModelState.IsValid)
            {
                db.Comunicaciones.AddObject(anuncio);
                db.SaveChanges();
                return RedirectToAction("Index", new { privados = anuncio.Privada });
            }

            ViewBag.Privado = anuncio.Privada;
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", anuncio.IdCategoria);
            return View(anuncio);
        }

        //
        // GET: /Anuncios/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Anuncio anuncio = db.Comunicaciones.OfType<Anuncio>().Single(a => a.IdComunicacion == id);
            if (anuncio == null)
            {
                return HttpNotFound();
            }

            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", anuncio.IdCategoria); 
            return View(anuncio);
        }

        //
        // POST: /Anuncios/Edit/5

        [HttpPost]
        public ActionResult Edit(Anuncio anuncio)
        {
            if (ModelState.IsValid)
            {
                db.Comunicaciones.Attach(anuncio);
                db.ObjectStateManager.ChangeObjectState(anuncio, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index", new { privados = anuncio.Privada });
            }

            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", anuncio.IdCategoria);
            return View(anuncio);
        }

        //
        // GET: /Anuncios/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Anuncio anuncio = db.Comunicaciones.OfType<Anuncio>().Single(a => a.IdComunicacion == id);
            if (anuncio == null)
            {
                return HttpNotFound();
            }
            return View(anuncio);
        }

        //
        // POST: /Anuncios/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Anuncio anuncio = db.Comunicaciones.OfType<Anuncio>().Single(a => a.IdComunicacion == id);
            bool privado = (bool) anuncio.Privada;
            db.Comunicaciones.DeleteObject(anuncio);
            db.SaveChanges();
            return RedirectToAction("Index", new { privados = privado });
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        public ActionResult Comunidades(int id)
        {
            return RedirectToAction("Index", "ComunicacionesComunidades", new { idComunicacion = id });
        }

        public ActionResult Comentarios(int id)
        {
            return RedirectToAction("Index", "ComentariosAnuncios", new { idAnuncio = id });
        }
    }
}