﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Threading;
using System.Globalization;
using System.Web;
using TFC.Filters;
using TFC.Helpers;

namespace TFC.Controllers
{
    [Localization]
    public abstract class BaseController : Controller
    {
        protected override void ExecuteCore()
        {
            if (RouteData.Values["lang"] != null &&
                !string.IsNullOrWhiteSpace(RouteData.Values["lang"].ToString()))
            {
                // set the culture from the route data (url)
                var lang = RouteData.Values["lang"].ToString();
                Thread.CurrentThread.CurrentUICulture = CultureInfo.CreateSpecificCulture(lang);
            }
            else
            {
                // load the culture info from the cookie
                var cookie = HttpContext.Request.Cookies["TFC.CurrentUICulture"];
                var langHeader = string.Empty;
                if (cookie != null)
                {
                    // set the culture by the cookie content
                    langHeader = cookie.Value;
                    Thread.CurrentThread.CurrentUICulture = CultureInfo.CreateSpecificCulture(langHeader);
                }
                else
                {
                    // set the culture by the location if not speicified
                    langHeader = HttpContext.Request.UserLanguages[0];
                    Thread.CurrentThread.CurrentUICulture = CultureInfo.CreateSpecificCulture(langHeader);
                }
                // set the lang value into route data
                RouteData.Values["lang"] = langHeader;
            }

            // save the location into cookie
            HttpCookie _cookie = new HttpCookie("TFC.CurrentUICulture", Thread.CurrentThread.CurrentUICulture.Name);
            _cookie.Expires = DateTime.Now.AddYears(1);
            HttpContext.Response.SetCookie(_cookie);

            base.ExecuteCore();
        }

        /// <summary>
        /// Método que se llama desde los controladores que heredan de BaseController para obtener
        /// el menú lateral del menú principal que se pasa como parámetro. Los elementos serán seleccionados
        /// según el módulo o menú principal seleccionado y según el rol del usuario autenticado.
        /// </summary>
        /// <param name="modulo"></param>
        /// <returns>Lista con los elementos de menú del módulo en cuestión.</returns>
        protected List<MenuItem> GetMenu(string menu)
        {
            List<MenuItem> items = new List<MenuItem>();

            if (Request.IsAuthenticated)
            {
                if (menu == "DatosMaestros")
                {
                    if (User.IsInRole("Administrador"))
                    {
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.AdminsFincas, "Index", "AdministradorFincas"));
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.TiposDePropiedad, "Index", "TiposPropiedad"));
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.CargosComunidad, "Index", "Cargos"));
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.Paises, "Index", "Paises"));
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.Provincias, "Index", "Provincias"));
                    }
                    else if (User.IsInRole("Administrador de fincas"))
                    {
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.Comunidades, "Index", "Comunidades"));
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.Propietarios, "Index", "Propietario"));
                        items.Add(new MenuItem(Resources.Sidebar.DatosMaestros.EjerciciosCargos, "Index", "EjerciciosCargos"));
                    }
                }
                if (menu == "Comunicaciones")
                {
                    if (User.IsInRole("Administrador"))
                    {
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.EstadosIncidencias, "Index", "EstadosComunicacion"));
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.CategoriasIncidencias, "Index", "CategoriasComunicacion"));

                    }
                    else
                    {
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.Incidencias, "Index", "Incidencias"));
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.Noticias, "Index", "Noticias"));
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.Anuncios, "Index", "Anuncios"));
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.Sugerencias, "Index", "Sugerencias"));
                        items.Add(new MenuItem(Resources.Sidebar.Comunicaciones.Biblioteca, "Index", "Biblioteca"));
                    }
                }
                if (menu == "Contabilidad")
                {
                    if (Request.IsAuthenticated)
                    {
                        if (User.IsInRole("Administrador"))
                        {
                            items.Add(new MenuItem(Resources.Sidebar.Contabilidad.Grupos, "Index", "Grupos"));
                            items.Add(new MenuItem(Resources.Sidebar.Contabilidad.Cuentas, "Index", "Cuentas"));
                            items.Add(new MenuItem(Resources.Sidebar.Contabilidad.FormasCobro, "Index", "FormasCobro"));
                        }
                        else if (User.IsInRole("Administrador de fincas"))
                        {
                            items.Add(new MenuItem(Resources.Sidebar.Contabilidad.Subcuentas, "Index", "Subcuentas"));
                            items.Add(new MenuItem(Resources.Sidebar.Contabilidad.GestionDeRecibos, "Index", "Recibos"));
                            items.Add(new MenuItem(Resources.Sidebar.Contabilidad.DiarioContable, "EnConstruccion", "Error"));
                        }
                        else
                        {
                            
                        }
                    }
                }
                if (menu == "Informes")
                {
                    if (User.IsInRole("Administrador"))
                    {

                    }
                    else
                    {
                        items.Add(new MenuItem(Resources.Sidebar.Informes.IngresosYGastos, "EnConstruccion", "Error"));
                        items.Add(new MenuItem(Resources.Sidebar.Informes.EstadoComunidad, "EnConstruccion", "Error"));
                    }
                }
                if (menu == "Configuracion")
                {
                    if (Request.IsAuthenticated)
                    {
                        if (User.IsInRole("Administrador"))
                        {
                            items.Add(new MenuItem("Parámetros", "Index", "Parametros"));
                        }
                    }
                }
            }
            return items;
        }
    }
}
