﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TFC.Models;
using System.Transactions;
using TFC.Helpers;

namespace TFC.Controllers
{
    public class IncidenciasController : BaseController
    {
        private TFCEntities1 db = new TFCEntities1();

        //
        // GET: /Incidencias/

        public ActionResult Index()
        {
            IEnumerable<ListaIncidencias> incidencias = null;

            if (User.IsInRole("Propietario"))
            {                            
                // Si es un propietario, mostramos las incidencias de las comunidades donde tiene alguna propiedad.
                incidencias = from co in db.Comunicaciones.OfType<Incidencia>()
                                        .Include("Persona")
                                        .Include("CategoriaComunicacion")
                                        .Include("EstadoComunicacion")
                                join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion 
                                join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                                join p in db.Propiedades on c.IdComunidad equals p.IdComunidad
                                join pp in db.PropiedadesPropietarios on p.IdPropiedad equals pp.IdPropiedad
                                join po in db.Personas.OfType<Propietario>() on pp.PropietarioIdPersona equals po.IdPersona 
                                where po.Email == User.Identity.Name
                                orderby co.IdComunicacion 
                              select new ListaIncidencias
                                {
                                    IdComunicacion = co.IdComunicacion,
                                    IdComunidad = c.IdComunidad,
                                    ComunidadDesc = c.Direccion + " (" + c.Localidad + ")",
                                    Fecha = co.Fecha,
                                    Persona = (co.Persona.Nombre == null ? "" : co.Persona.Nombre) + " " + (co.Persona.Apellidos == null ? "" : co.Persona.Apellidos),
                                    Categoria = co.CategoriaComunicacion.Descripcion,
                                    Estado = co.EstadoComunicacion.Descripcion
                                }; 
            }
            else if (User.IsInRole("Administrador de fincas"))
            {
                // Si es un administrador de fincas, mostramos las incidencias de las comunidades clientes.
                incidencias = from co in db.Comunicaciones.OfType<Incidencia>()
                                    .Include("Persona")
                                    .Include("CategoriaComunicacion")
                                    .Include("EstadoComunicacion")
                                join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                                join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                                join a in db.Personas on c.AdministradorFincasIdPersona equals a.IdPersona 
                                where a.Email == User.Identity.Name
                              select new ListaIncidencias
                              {
                                  IdComunicacion = co.IdComunicacion,
                                  IdComunidad = c.IdComunidad,
                                  ComunidadDesc = c.Direccion + " (" + c.Localidad + ")",
                                  Fecha = co.Fecha,
                                  Persona = (co.Persona.Nombre == null ? "" : co.Persona.Nombre) + " " + (co.Persona.Apellidos == null ? "" : co.Persona.Apellidos),
                                  Categoria = co.CategoriaComunicacion.Descripcion,
                                  Estado = co.EstadoComunicacion.Descripcion
                              }; 
            }

        
            ViewBag.IdPersona = db.Personas.Single(p => p.Email == User.Identity.Name).IdPersona;
                                       
            return View(incidencias.ToList());
        }

        //
        // GET: /Incidencias/Details/5

        public ActionResult Details(int id = 0)
        {
            Incidencia incidencia = db.Comunicaciones.OfType<Incidencia>().Single(n => n.IdComunicacion == id);
            if (incidencia == null)
            {
                return HttpNotFound();
            }
            return View(incidencia);
        }


        //
        // GET: /Incidencias/Create

        public ActionResult Create()
        {
            Persona persona = Entorno.PersonaConectada(User.Identity.Name);
           
            ViewBag.NombrePersona = persona.Nombre + " " + persona.Apellidos;

            this.GetComunidades();
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion");
            ViewBag.IdEstado = new SelectList(db.EstadosComunicacion, "IdEstado", "Descripcion", 1);
            Incidencia incidencia = new Incidencia { IdPersona = persona.IdPersona, IdEstado = 1 };
            return View(incidencia);
        }

        //
        // POST: /Incidencias/Create

        [HttpPost]
        public ActionResult Create(Incidencia incidencia)
        {
            bool exito = false;

            // Grabamos la hora actual
            incidencia.Fecha = DateTime.Now;

            if (ModelState.IsValid)
            {
                using (TransactionScope transaction = new TransactionScope())
                {
                    try
                    {
                        db.Comunicaciones.AddObject(incidencia);
                        db.SaveChanges();

                        //Grabamos la comunidad
                        ComunicacionComunidad cc = new ComunicacionComunidad();
                        cc.IdComunicacion = incidencia.IdComunicacion;
                        cc.IdComunidad = incidencia.IdComunidad;
                        db.ComunicacionesComunidades.AddObject(cc);
                        db.SaveChanges();

                        transaction.Complete();
                        exito = true;
                    }
                    catch (Exception ex)
                    {

                    }

                    if (exito)
                    {
                        db.AcceptAllChanges();
                    }
                }

                return RedirectToAction("Index");
             }

            this.GetComunidades();
            Persona persona = Entorno.PersonaConectada(User.Identity.Name);
            ViewBag.IdPersona = persona.IdPersona;
            ViewBag.NombrePersona = persona.Nombre + " " + persona.Apellidos;
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", incidencia.IdCategoria);
            ViewBag.IdEstado = new SelectList(db.EstadosComunicacion, "IdEstado", "Descripcion", incidencia.IdEstado);
            return View(incidencia);
        }

        //
        // GET: /Incidencias/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Incidencia incidencia = db.Comunicaciones.OfType<Incidencia>().Single(i => i.IdComunicacion == id);
            if (incidencia == null)
            {
                return HttpNotFound();
            }

            this.GetComunidades(incidencia.IdComunidad);
            Persona persona = Entorno.PersonaConectada(User.Identity.Name);
            ViewBag.IdPersona = persona.IdPersona;
            ViewBag.NombrePersona = persona.Nombre + " " + persona.Apellidos;
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", incidencia.IdCategoria);
            ViewBag.IdEstado = new SelectList(db.EstadosComunicacion, "IdEstado", "Descripcion", incidencia.IdEstado);
            return View(incidencia);
        }

        //
        // POST: /Incidencias/Edit/5

        [HttpPost]
        public ActionResult Edit(Incidencia incidencia)
        {
            if (ModelState.IsValid)
            {
                db.Comunicaciones.Attach(incidencia);
                db.ObjectStateManager.ChangeObjectState(incidencia, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            ViewBag.IdPersona = new SelectList(db.Personas, "IdPersona", "Nombre", incidencia.IdPersona);
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", incidencia.IdCategoria);
            ViewBag.IdEstado = new SelectList(db.EstadosComunicacion, "IdEstado", "Descripcion", incidencia.IdEstado);
            return View(incidencia);
        }

        //
        // GET: /Incidencias/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Incidencia incidencia = db.Comunicaciones.OfType<Incidencia>().Single(i => i.IdComunicacion == id);
            if (incidencia == null)
            {
                return HttpNotFound();
            }
            return View(incidencia);
        }

        //
        // POST: /Incidencias/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Incidencia incidencia = db.Comunicaciones.OfType<Incidencia>().Single(i => i.IdComunicacion == id);
            db.Comunicaciones.DeleteObject(incidencia);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        private void GetComunidades(int idComunidad = 0)
        {
            // Obtenemos las comunidades a las que tiene acceso el usuario conectado.
            IList<Comunidad> listaComunidades = Datos.GetComunidades(User.Identity.Name);
            var comunidades = from c in listaComunidades
                              orderby c.Localidad, c.Direccion
                              select new
                              {
                                  IdComunidad = c.IdComunidad,
                                  Direccion = c.Direccion + " (" + c.Localidad + ")"
                              };

            if (idComunidad > 0)
            {
                //Si el parámetro opcional idComunidad tiene valor, se lo asociamos a la lista como valor seleccionado por defecto.
                ViewBag.IdComunidad = new SelectList(comunidades, "IdComunidad", "Direccion", idComunidad);
            }
            else
            {
                ViewBag.IdComunidad = new SelectList(comunidades, "IdComunidad", "Direccion");
            }
        }

        public ActionResult Comentarios(int id)
        {
            return RedirectToAction("Index", "ComentariosIncidencias", new { idIncidencia = id });
        }
    }
}