﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TFC.Models;
using System.Transactions;
using TFC.Helpers;

namespace TFC.Controllers
{
    public class NoticiasController : BaseController
    {
        private TFCEntities1 db = new TFCEntities1();

        //
        // GET: /Noticias/
        public ActionResult Index(bool privadas = false)
        {
            IEnumerable<Noticia> noticias = null;

            if (privadas == true)
            {
                // Obtenemos las comunidades del usuario conectado
                IList<Comunidad> comunidades = Datos.GetComunidades(User.Identity.Name);

                // Noticias privadas
                if (User.IsInRole("Propietario"))
                {
                    // Si es un propietario, mostramos las noticias privadas de las comunidades donde tiene alguna propiedad
                    noticias = (from co in db.Comunicaciones.OfType<Noticia>()
                                                                .Include("CategoriaComunicacion")
                                join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                                join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                                join p in db.Propiedades on c.IdComunidad equals p.IdComunidad
                                join pp in db.PropiedadesPropietarios on p.IdPropiedad equals pp.IdPropiedad
                                join po in db.Personas.OfType<Propietario>() on pp.PropietarioIdPersona equals po.IdPersona
                                where po.Email == User.Identity.Name && co.Privada == true && (co.FechaInicio <= DateTime.Today && co.FechaFin >= DateTime.Today)
                                orderby co.Fecha descending
                                select co).Distinct();

                }
                else if (User.IsInRole("Administrador de fincas"))
                {
                    // Si es un administrador de fincas, mostramos todas las noticias privadas de sus comunidades.
                    noticias = (from co in db.Comunicaciones.OfType<Noticia>()
                                        .Include("CategoriaComunicacion")
                               join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                               join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                               join af in db.Personas.OfType<AdministradorFincas>() on c.AdministradorFincasIdPersona equals af.IdPersona
                               where af.Email == User.Identity.Name && co.Privada == true
                               orderby co.Fecha descending
                               select co).Distinct();
                }      
            }
            else
            {
                // Noticias públicas. Son visibles por todos los usuarios de la aplicación.
                if (User.IsInRole("Propietario"))
                {
                    noticias = from co in db.Comunicaciones.OfType<Noticia>()
                                            .Include("CategoriaComunicacion")
                               where co.Privada == false && (co.FechaInicio <= DateTime.Today && co.FechaFin >= DateTime.Today)
                               orderby co.Fecha descending
                               select co;
                }
                else
                {
                    noticias = from co in db.Comunicaciones.OfType<Noticia>()
                                            .Include("CategoriaComunicacion")
                               where co.Privada == false 
                               orderby co.Fecha descending
                               select co;
                }
            }

            ViewBag.Privadas = privadas;
            ViewBag.IdPersona = Entorno.PersonaConectada(User.Identity.Name).IdPersona;
            return View(noticias.ToList());
        }

        //
        // GET: /Noticias/Details/5

        public ActionResult Details(int id = 0)
        {
            Noticia noticia = db.Comunicaciones.OfType<Noticia>().Single(n => n.IdComunicacion == id);
            if (noticia == null)
            {
                return HttpNotFound();
            }
            return View(noticia);
        }

        //
        // GET: /Noticias/Create

        public ActionResult Create(bool privada)
        {
            ViewBag.Privada = privada;
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion");

            // Ponemos valores por defecto
            Noticia noticia = new Noticia { EnviarNotificacion = privada, Privada = privada };

            return View(noticia);
        }

        //
        // POST: /Noticias/Create

        [HttpPost]
        public ActionResult Create(Noticia noticia)
        {
            noticia.IdPersona = Entorno.PersonaConectada(User.Identity.Name).IdPersona;
            noticia.Fecha = DateTime.Now;
            
            if (ModelState.IsValid)
            {
                db.Comunicaciones.AddObject(noticia);
                db.SaveChanges();
                return RedirectToAction("Index", new { privadas = noticia.Privada });
            }
            ViewBag.Privada = noticia.Privada;
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", noticia.IdCategoria);
 
            return View(noticia);
        }

        //
        // GET: /Noticias/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Noticia noticia = db.Comunicaciones.OfType<Noticia>().Single(n => n.IdComunicacion == id);
            if (noticia == null)
            {
                return HttpNotFound();
            }

            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", noticia.IdCategoria);
            return View(noticia);
        }

        //
        // POST: /Noticias/Edit/5

        [HttpPost]
        public ActionResult Edit(Noticia noticia)
        {
            if (ModelState.IsValid)
            {
                db.Comunicaciones.Attach(noticia);
                db.ObjectStateManager.ChangeObjectState(noticia, EntityState.Modified);
                db.SaveChanges();

                return RedirectToAction("Index", new { privadas = noticia.Privada });
 
            }
            ViewBag.IdCategoria = new SelectList(db.CategoriasComunicacion, "IdCategoria", "Descripcion", noticia.IdCategoria);
            return View(noticia);
        }

        //
        // GET: /Noticias/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Noticia noticia = db.Comunicaciones.OfType<Noticia>().Single(n => n.IdComunicacion == id);
            if (noticia == null)
            {
                return HttpNotFound();
            }
            ViewBag.Privada = noticia.Privada;
            return View(noticia);
        }

        //
        // POST: /Noticias/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Noticia noticia = db.Comunicaciones.OfType<Noticia>().Single(n => n.IdComunicacion == id);

            bool privada = (bool) noticia.Privada;

            db.Comunicaciones.DeleteObject(noticia);
            db.SaveChanges();

            return RedirectToAction("Index", new { privadas = privada });
 
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        public ActionResult Comunidades(int id)
        {
            return RedirectToAction("Index", "ComunicacionesComunidades", new { idComunicacion = id });
        }

        public ActionResult Comentarios(int id)
        {
            return RedirectToAction("Index", "ComentariosNoticias", new { idNoticia = id });
        }
    }
}