﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using TFC.Models;
using System.Transactions;
using TFC.Helpers;

namespace TFC.Controllers
{
    public class SugerenciasController : BaseController
    {
        private TFCEntities1 db = new TFCEntities1();

        //
        // GET: /Sugerencias/

        public ActionResult Index()
        {
            IEnumerable<ListaSugerencias> sugerencias = null;

            if (User.IsInRole("Propietario"))
            {
                // Si es un propietario, mostramos las incidencias de las comunidades donde tiene alguna propiedad.
                sugerencias = from co in db.Comunicaciones.OfType<Sugerencia>()
                                        .Include("Persona")
                              join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                              join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                              join p in db.Propiedades on c.IdComunidad equals p.IdComunidad
                              join pp in db.PropiedadesPropietarios on p.IdPropiedad equals pp.IdPropiedad
                              join po in db.Personas.OfType<Propietario>() on pp.PropietarioIdPersona equals po.IdPersona
                              where po.Email == User.Identity.Name
                              orderby co.Fecha descending
                              select new ListaSugerencias
                              {
                                  IdComunicacion = co.IdComunicacion,
                                  ComunidadDesc = c.Direccion + " (" + c.Localidad + ")",
                                  Fecha = co.Fecha,
                                  Persona = co.Persona.Nombre + " " + co.Persona.Apellidos,
                                  PropiedadDesc = p.TipoPropiedad.Tipo + " - " + p.PropiedadDesc,
                                  Texto = co.Texto,
                                  Email = co.Persona.Email
                              };
            }
            else if (User.IsInRole("Administrador de fincas"))
            {
                // Si es un administrador de fincas, mostramos las incidencias de las comunidades clientes.
                sugerencias = from co in db.Comunicaciones.OfType<Sugerencia>()
                                    .Include("Persona")
                              join cc in db.ComunicacionesComunidades on co.IdComunicacion equals cc.IdComunicacion
                              join c in db.Comunidades on cc.IdComunidad equals c.IdComunidad
                              join af in db.Personas.OfType<AdministradorFincas>() on c.AdministradorFincasIdPersona equals af.IdPersona
                              where af.Email == User.Identity.Name 
                              orderby co.Fecha descending 
                              select new ListaSugerencias
                              {
                                  IdComunicacion = co.IdComunicacion,
                                  ComunidadDesc = c.Direccion + " (" + c.Localidad + ")",
                                  Fecha = co.Fecha,
                                  Persona = co.Persona.Nombre + " " + co.Persona.Apellidos,
                                  PropiedadDesc = string.Empty,
                                  Texto = co.Texto,
                                  Email = co.Persona.Email
                              };
            }


            ViewBag.IdPersona = Entorno.PersonaConectada(User.Identity.Name).IdPersona;

            return View(sugerencias.ToList());
        }

        //
        // GET: /Sugerencias/Create

        public ActionResult Create()
        {
            Persona persona = Entorno.PersonaConectada(User.Identity.Name);
            ViewBag.IdPersona = persona.IdPersona;
            ViewBag.NombrePersona = persona.Nombre + " " + persona.Apellidos;

            this.GetComunidades();
            return View();
        }

        //
        // POST: /Sugerencias/Create

        [HttpPost]
        public ActionResult Create(Sugerencia sugerencia)
        {
            bool exito = false;

            sugerencia.Fecha = DateTime.Now;

            if (ModelState.IsValid)
            {
                using (TransactionScope transaction = new TransactionScope())
                {
                    try
                    {
                        db.Comunicaciones.AddObject(sugerencia);
                        db.SaveChanges();

                        //Grabamos la comunidad
                        ComunicacionComunidad cc = new ComunicacionComunidad();
                        cc.IdComunicacion = sugerencia.IdComunicacion;
                        cc.IdComunidad = sugerencia.IdComunidad;
                        db.ComunicacionesComunidades.AddObject(cc);
                        db.SaveChanges();

                        transaction.Complete();
                        exito = true;
                    }
                    catch (Exception ex)
                    {

                    }

                    if (exito)
                    {
                        db.AcceptAllChanges();
                    }
                }

                return RedirectToAction("Index");
            }

            this.GetComunidades();
            Persona persona = Entorno.PersonaConectada(User.Identity.Name);
            ViewBag.IdPersona = persona.IdPersona;
            ViewBag.NombrePersona = persona.Nombre + " " + persona.Apellidos;
            return View(sugerencia);
        }

        //
        // GET: /Sugerencias/Edit/5

        public ActionResult Edit(int id = 0)
        {
            Sugerencia sugerencia = db.Comunicaciones.OfType<Sugerencia>().Single(s => s.IdComunicacion == id);
            if (sugerencia == null)
            {
                return HttpNotFound();
            }

            this.GetComunidades(sugerencia.IdComunidad);
            Persona persona = Entorno.PersonaConectada(User.Identity.Name);
            ViewBag.IdPersona = persona.IdPersona;
            ViewBag.NombrePersona = persona.Nombre + " " + persona.Apellidos;

            return View(sugerencia);
        }

        //
        // POST: /Sugerencias/Edit/5

        [HttpPost]
        public ActionResult Edit(Sugerencia sugerencia)
        {
            if (ModelState.IsValid)
            {
                db.Comunicaciones.Attach(sugerencia);
                db.ObjectStateManager.ChangeObjectState(sugerencia, EntityState.Modified);
                db.SaveChanges();
                return RedirectToAction("Index");
            }

            return View(sugerencia);
        }

        //
        // GET: /Sugerencias/Delete/5

        public ActionResult Delete(int id = 0)
        {
            Sugerencia sugerencia = db.Comunicaciones.OfType<Sugerencia>().Single(s => s.IdComunicacion == id);
            if (sugerencia == null)
            {
                return HttpNotFound();
            }
            return View(sugerencia);
        }

        //
        // POST: /Sugerencias/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Sugerencia sugerencia = db.Comunicaciones.OfType<Sugerencia>().Single(s => s.IdComunicacion == id);
            db.Comunicaciones.DeleteObject(sugerencia);
            db.SaveChanges();
            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        private void GetComunidades(int idComunidad = 0)
        {
            // Obtenemos las comunidades a las que tiene acceso el usuario conectado.
            IList<Comunidad> listaComunidades = Datos.GetComunidades(User.Identity.Name);
            var comunidades = from c in listaComunidades
                              orderby c.Localidad, c.Direccion
                              select new
                              {
                                  IdComunidad = c.IdComunidad,
                                  Direccion = c.Direccion + " (" + c.Localidad + ")"
                              };

            if (idComunidad > 0)
            {
                //Si el parámetro opcional idComunidad tiene valor, se lo asociamos a la lista como valor seleccionado por defecto.
                ViewBag.IdComunidad = new SelectList(comunidades, "IdComunidad", "Direccion", idComunidad);
            }
            else
            {
                ViewBag.IdComunidad = new SelectList(comunidades, "IdComunidad", "Direccion");
            }

        }

        public ActionResult Comentarios(int id)
        {
            return RedirectToAction("Index", "ComentariosSugerencias", new { idSugerencia = id });
        }
    }
}